/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.finance.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.finance.entity.PlatformAuthorization;
import org.springblade.modules.finance.wrapper.PlatformAuthorizationWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.finance.vo.PlatformAuthorizationVO;
import org.springblade.modules.finance.service.IPlatformAuthorizationService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 *  控制器
 *
 * @author BladeX
 * @since 2022-04-09
 */
@RestController
@AllArgsConstructor
@RequestMapping("finance/platformauthorization")
@Api(value = "平台授权表", tags = "接口")
public class PlatformAuthorizationController extends BladeController {

	private final IPlatformAuthorizationService platformAuthorizationService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入platformAuthorization")
	public R<PlatformAuthorization> detail(PlatformAuthorization platformAuthorization) {
		PlatformAuthorization detail = platformAuthorizationService.getOne(Condition.getQueryWrapper(platformAuthorization));
		return R.data(detail);
	}

	/**
	 * 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入platformAuthorization")
	public R<IPage<PlatformAuthorizationVO>> list(PlatformAuthorization platformAuthorization, Query query) {
		IPage<PlatformAuthorization> pages = platformAuthorizationService.page(Condition.getPage(query), Condition.getQueryWrapper(platformAuthorization));
		return R.data(PlatformAuthorizationWrapper.build().pageVO(pages));
	}

	/**
	 * 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入platformAuthorization")
	public R<IPage<PlatformAuthorizationVO>> page(PlatformAuthorizationVO platformAuthorization, Query query) {
		IPage<PlatformAuthorizationVO> pages = platformAuthorizationService.selectPlatformAuthorizationPage(Condition.getPage(query), platformAuthorization);
		return R.data(pages);
	}

	/**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入platformAuthorization")
	public R save(@Valid @RequestBody PlatformAuthorization platformAuthorization) {
		return R.status(platformAuthorizationService.save(platformAuthorization));
	}

	/**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入platformAuthorization")
	public R update(@Valid @RequestBody PlatformAuthorization platformAuthorization) {
		return R.status(platformAuthorizationService.updateById(platformAuthorization));
	}

	/**
	 * 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入platformAuthorization")
	public R submit(@Valid @RequestBody PlatformAuthorization platformAuthorization) {
		return R.status(platformAuthorizationService.saveOrUpdate(platformAuthorization));
	}


	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(platformAuthorizationService.removeByIds(Func.toLongList(ids)));
	}


}
