/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.shopping.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.shopping.wrapper.GoodsWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.shopping.entity.Goods;
import org.springblade.modules.shopping.vo.GoodsVO;
import org.springblade.modules.shopping.service.IGoodsService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 商品表 控制器
 *
 * @author BladeX
 * @since 2022-02-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("shopping/goods")
@Api(value = "商品表", tags = "商品表接口")
public class GoodsController extends BladeController {

	private final IGoodsService goodsService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入goods")
	public R<Goods> detail(Goods goods) {
		Goods detail = goodsService.getOne(Condition.getQueryWrapper(goods));
		return R.data(detail);
	}

	/**
	 * 分页 商品表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入goods")
	public R<IPage<GoodsVO>> list(Goods goods, Query query) {
		IPage<Goods> pages = goodsService.page(Condition.getPage(query), Condition.getQueryWrapper(goods));
		return R.data(GoodsWrapper.build().pageVO(pages));
	}

	/**
	 * 自定义分页 商品表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入goods")
	public R<IPage<GoodsVO>> page(GoodsVO goods, Query query) {
		IPage<GoodsVO> pages = goodsService.selectGoodsPage(Condition.getPage(query), goods);
		return R.data(pages);
	}

	/**
	 * 新增 商品表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入goods")
	public R save(@Valid @RequestBody Goods goods) {
		return R.status(goodsService.save(goods));
	}

	/**
	 * 修改 商品表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入goods")
	public R update(@Valid @RequestBody Goods goods) {
		return R.status(goodsService.updateById(goods));
	}

	/**
	 * 新增或修改 商品表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入goods")
	public R submit(@Valid @RequestBody Goods goods) {
		return R.status(goodsService.saveOrUpdate(goods));
	}


	/**
	 * 删除 商品表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(goodsService.removeByIds(Func.toLongList(ids)));
	}


}
