/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.shopping.wrapper;

import org.springblade.common.cache.DictCache;
import org.springblade.common.enums.DictEnum;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.modules.shopping.entity.Category;
import org.springblade.modules.shopping.entity.Goods;
import org.springblade.modules.shopping.entity.SpecCats;
import org.springblade.modules.shopping.entity.SpecItems;
import org.springblade.modules.shopping.service.ICategoryService;
import org.springblade.modules.shopping.service.IGoodsService;
import org.springblade.modules.shopping.service.ISpecCatsService;
import org.springblade.modules.shopping.vo.SpecCatsVO;
import org.springblade.modules.shopping.vo.SpecItemsVO;

import java.util.Objects;

/**
 * 包装类,返回视图层所需的字段
 *
 * @author Chill
 */
public class SpecItemsWrapper extends BaseEntityWrapper<SpecItems, SpecItemsVO> {

	public static SpecItemsWrapper build() {
		return new SpecItemsWrapper();
	}

	public static IGoodsService goodsService;

	public static ISpecCatsService specCatsService;

	static {
		goodsService = SpringUtil.getBean(IGoodsService.class);

		specCatsService = SpringUtil.getBean(ISpecCatsService.class);
	}

	@Override
	public SpecItemsVO entityVO(SpecItems entity) {
		SpecItemsVO vo = Objects.requireNonNull(BeanUtil.copy(entity, SpecItemsVO.class));
		Goods goods = goodsService.getById(entity.getGoodsId());
		if (goods != null){
			vo.setGoodsName(goods.getGoodsName());
			vo.setProductNo(goods.getProductNo());
		}
		SpecCats specCats = specCatsService.getById(entity.getSpecCatId());
		if (specCats != null){
			vo.setSpecName(specCats.getSpecName());
		}
		return vo;
	}

}
